@@ -1,4 +1,4 @@
-// $Id: cfg.c,v 1.13 2002/08/28 22:20:26 ensc Exp $    --*- c++ -*--
+// $Id: cfg.c,v 1.18 2004/06/16 10:47:36 ensc Exp $    --*- c++ -*--
 
 // Copyright (C) 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -113,17 +113,22 @@ initRawFD(/*@out@*/int *fd)
   *fd = Esocket(AF_PACKET, SOCK_RAW, 0xFFFF);
 }
 
-inline static void
-initSenderFD(/*@out@*/int *fd)
+inline static int
+initSenderFD(struct InterfaceInfo const *iface)
     /*@globals internalState, fileSystem@*/
     /*@modifies internalState, fileSystem, *fd@*/
-    /*@requires maxRead(fd)==0 /\ maxSet(fd)==0@*/
 {
   struct sockaddr_in	s;
+  int			fd;
+  int const		ON = 1;
 
-  assert(fd!=0);
+  fd = Esocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
 
-  *fd = Esocket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+    // used for sending only...
+  Esetsockopt(fd, SOL_SOCKET, SO_REUSEADDR, &ON, sizeof ON);
+  if (iface)
+    Esetsockopt(fd, SOL_SOCKET, SO_BINDTODEVICE,
+		iface->name, strlen(iface->name)+1);
 
   memset(&s, 0, sizeof(s));
   
@@ -132,7 +137,8 @@ initSenderFD(/*@out@*/int *fd)
   s.sin_port        = htons(DHCP_PORT_CLIENT);
   s.sin_addr.s_addr = htonl(INADDR_ANY);
 
-  Ebind(*fd, &s);
+  Ebind(fd, &s);
+  return fd;
 }
 
 inline static void
@@ -205,6 +211,27 @@ fillInterfaceInfo(struct InterfaceInfoList *ifs)
   scEXITFATAL("Can not get interface information");
 }
 
+static char const *
+getSenderIfaceName(struct InterfaceInfoList const * const ifs,
+		   bool                    		  do_it)
+{
+  struct InterfaceInfo const *	res = 0;
+  struct InterfaceInfo const *	ptr = ifs->len==0 ? 0 : ifs->dta + ifs->len;
+  
+  if (!do_it) return 0;
+
+  while (ptr>ifs->dta) {
+    --ptr;
+    if (ptr->has_servers) {
+      if (res) return 0;	// there are more than one sender...
+      else     res = ptr;
+    }
+  }
+
+  if (res) return res->name;
+  else     return 0;
+}
+
 inline static void
 initFDs(/*@out@*/struct FdInfoList 		*fds,
 	/*@in@*/struct ConfigInfo const	* const	cfg)
@@ -214,8 +241,35 @@ initFDs(/*@out@*/struct FdInfoList 		*fds,
 {
   size_t					i, idx;
   struct InterfaceInfoList const * const	ifs = &cfg->interfaces;
+  struct ServerInfo *				servers;
+  int						bind_all_fd = -1;
+
+  for (servers = cfg->servers.dta;
+       servers < cfg->servers.dta + cfg->servers.len;
+       ++servers) {
+    switch (servers->type) {
+      case svUNICAST	:
+	if (bind_all_fd!=-1 && servers->iface!=0)
+	  scEXITFATAL("There are mixed 'server ip ...' declarations; please use either only such ones with or such ones without an interface");
+	else if (bind_all_fd!=-1)
+	  servers->info.unicast.fd = bind_all_fd;
+	else if (servers->iface!=0 && servers->iface->sender_fd!=-1)
+	  servers->info.unicast.fd = servers->iface->sender_fd;
+	else {
+	  servers->info.unicast.fd = initSenderFD(servers->iface);
+
+	  if (servers->iface==0)
+	    bind_all_fd               = servers->info.unicast.fd;
+	  else
+	    servers->iface->sender_fd = servers->info.unicast.fd;
+	}
+
+	break;
+      default		:
+	break;
+    }
+  }
 
-  initSenderFD(&fds->sender_fd);
   initRawFD(&fds->raw_fd);
   
   fds->len = ifs->len;
@@ -283,7 +337,7 @@ showVersion() /*@*/
 inline static void
 showHelp(/*@in@*//*@nullterminated@*/char const *cmd) /*@*/
 {
-  char const	msg[] = (" [-v] [-h] [-c <filename>] [-d]\n\n"
+  char const	msg[] = (" [-v] [-h] [-c <filename>] [-n] [-d]\n\n"
 			 "  -v              show version\n"
 			 "  -h              show help\n"
 			 "  -c <filename>   read configuration from <filename>\n"
@@ -331,7 +385,7 @@ initializeDaemon(/*@in@*/struct ConfigInfo const *cfg)
 {
   assert(cfg!=0);
   
-  if (cfg->do_fork) Esetsid();
+  if (cfg->do_fork) (void)Esetsid();
 
   Eclose(1);
       
@@ -357,7 +411,7 @@ parseCommandline(int argc, char *argv[],
   assert(cfg!=0);
 
   while (true) {
-    int c = getopt(argc, argv, "vhdnc:");
+    int c = getopt(argc, argv, "vhdnc:-");
     if (c==-1) break;
 
     switch (c) {
@@ -366,7 +420,12 @@ parseCommandline(int argc, char *argv[],
       case 'd'	:
       case 'n'	:  cfg->do_fork       = false;  break;
       case 'c'	:  cfg->conffile_name = optarg; break;
-      default	:  scEXITFATAL("Use '-h' to get help about possible options");
+      case '-'	:
+	if (strcmp(argv[optind], "--version")==0) { showVersion();     exit(0); }
+	if (strcmp(argv[optind], "--help")==0)    { showHelp(argv[0]); exit(0); }
+	  /*@fallthrough@*/
+      default	:
+	scEXITFATAL("Use '-h' to get help about possible options");
     }
   }
   
@@ -386,6 +445,7 @@ initializeSystem(int argc, char *argv[],
 
   cfg.conffile_name = CFG_FILENAME;
   cfg.do_fork       = true;
+  cfg.do_bindall    = false;
 
   parseCommandline(argc, argv, &cfg);
 
