@@ -1,4 +1,4 @@
-// $Id: dhcp.h,v 1.9 2002/07/11 16:44:20 ensc Exp $    --*- c++ -*--
+// $Id: dhcp.h,v 1.13 2002/08/29 15:39:53 ensc Exp $    --*- c++ -*--
 
 // Copyright (C) 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -19,13 +19,10 @@
 #ifndef DHCP_FORWARDER_SRC_DHCP_H
 #define DHCP_FORWARDER_SRC_DHCP_H
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <stdint.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
+#include <assert.h>
 
 #include "compat.h"
 #include "util.h"
@@ -126,6 +123,26 @@ DHCP_getOptionLength(/*@sef@*//*@in@*/struct DHCPSingleOption const *opt)
   }
 }
 
+/*@unused@*/
+inline static void
+DHCP_removeOption(struct DHCPSingleOption *opt,
+		  struct DHCPSingleOption const *end_opt)
+    /*@requires opt <= end_opt@*/
+    /*@modifies *opt@*/
+{
+  size_t		len = DHCP_getOptionLength(opt);
+  char * const		start = reinterpret_cast(char *)(opt);
+  char const * const	end   = reinterpret_cast(char const *)(end_opt);
+  
+  assert(opt <= end_opt);
+  if (start+len > end) return;	// TODO: broken option-list ... what to do?
+
+    // end-start  -->  character count between opt and end_opt without end_opt
+    // -len       -->  size of the option to be removed
+    // +1         -->  the end_opt
+  memmove(start, start+len, end-start - len + 1);
+}
+
 /*@unused@*/
 inline static void
 DHCP_zeroOption(struct DHCPSingleOption *opt)
