@@ -1,4 +1,4 @@
-// $Id: logging.h,v 1.2 2002/06/12 00:44:32 ensc Exp $    --*- c++ -*--
+// $Id: logging.h,v 1.5 2003/06/30 23:31:26 ensc Exp $    --*- c++ -*--
 
 // Copyright (C) 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -19,19 +19,19 @@
 #ifndef DHCP_FORWARDER_LOGGING_H
 #define DHCP_FORWARDER_LOGGING_H
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
 #include <netinet/in.h>
 
-#define LOG(MSG)	((void)write(2, MSG, sizeof(MSG)))
-#define LOGSTR(MSG)	((void)write(2, MSG, strlen(MSG)))
+#include "output.h"
 
-#ifdef ENABLE_LOGGING
-void logDHCPPackage(char const *buffer, size_t len,
-		    struct in_pktinfo const	*pkinfo,
-		    void const			*addr);
+#define LOG(MSG)	writeMsg(MSG, sizeof(MSG)-1)
+#define LOGSTR(MSG)	writeMsg(MSG, strlen(MSG))
+
+#ifdef WITH_LOGGING
+void logDHCPPackage(/*@in@*/char const *buffer, size_t len,
+		    /*@in@*/struct in_pktinfo const	*pkinfo,
+		    /*@in@*/void const			*addr)
+  /*@globals internalState@*/
+  /*@modifies internalState@*/ ;
 #endif
 
 #endif	//  DHCP_FORWARDER_LOGGING_H
