@@ -1,4 +1,4 @@
-// $Id: main.c,v 1.12 2002/07/11 17:02:20 ensc Exp $    --*- c++ -*--
+// $Id: main.c,v 1.19 2004/06/16 10:48:36 ensc Exp $    --*- c++ -*--
 
 // Copyright (C) 2002 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -35,10 +35,12 @@
 #include <sys/select.h>
 #include <sys/param.h>
 #include <errno.h>
-#include <netpacket/packet.h>
+#include <alloca.h>
 #include <net/if_arp.h>
 #include <sys/ioctl.h>
 
+#include "compat/if_packet.h"
+
 #include "cfg.h"
 #include "wrappers.h"
 #include "dhcp.h"
@@ -61,7 +63,7 @@ typedef enum {
 } OptionFillAction;
 
 /*@checkmod@*/static struct ServerInfoList	servers;
-/*@checkmod@*/static struct FdInfoList	fds;
+/*@checkmod@*/static struct FdInfoList		fds;
 
 inline static void
 fillFDSet(/*@out@*/fd_set			*fdset,
@@ -247,6 +249,7 @@ replaceAgentOption(/*@in@*/struct InterfaceInfo const * const	iface,
 		   struct DHCPSingleOption			*relay_opt,
 		   struct DHCPSingleOption			*end_opt,
 		   size_t					len)
+    /*@requires end_opt > relay_opt@*/
 {
   size_t	opt_len = DHCP_getOptionLength(relay_opt);
   size_t	str_len = strlen(iface->aid);
@@ -259,17 +262,41 @@ replaceAgentOption(/*@in@*/struct InterfaceInfo const * const	iface,
     memcpy(relay_opt->data+2, iface->aid, str_len);
 
     if (str_len+4<opt_len)
+	// TODO: move memory; do not pad to satisfy WinNT
       memset(relay_opt->data+2+str_len, /*@+charint@*/cdPAD/*@=charint@*/, opt_len-str_len-4);
   }
+  else if (opt_len>=len) {
+    DHCP_removeOption(relay_opt, end_opt);
+
+    len -= opt_len;
+    len  = addAgentOption(iface, end_opt, len);
+  }
   else {
-    DHCP_zeroOption(relay_opt);
-    len = addAgentOption(iface, end_opt, len);
+    LOGSTR("Failed assertion 'opt_len < len'");
   }
 
   return len;
 }
 #endif
 
+inline static size_t
+removeAgentOption(/*@dependent@*/struct DHCPSingleOption	*opt,
+		  struct DHCPSingleOption const 		*end_opt,
+		  size_t					len)
+    /*@requires end_opt > opt@*/
+    /*@modifies *opt@*/
+{
+  size_t	opt_len = DHCP_getOptionLength(opt);
+  
+  if (opt_len < len) {
+    DHCP_removeOption(opt, end_opt);
+    len -= opt_len;
+  }
+  else LOGSTR("Failed assertion 'opt_len < len'");
+
+  return len;
+}
+
   /*@-mustmod@*/
 inline static size_t
 fillOptions(/*@in@*/struct InterfaceInfo const* const	iface,
@@ -294,7 +321,8 @@ fillOptions(/*@in@*/struct InterfaceInfo const* const	iface,
     opt = DHCP_nextSingleOption(opt);
   } while (end_opt==0);
 
-  assert(end_opt>=option_ptr);
+  assert(static_cast(void*)(end_opt)>=option_ptr);
+  assert(end_opt>=relay_opt || relay_opt==0);
 
     /* Determine used space until end-tag and add space for the end-tag itself
      * (1 octet). */
@@ -305,7 +333,7 @@ fillOptions(/*@in@*/struct InterfaceInfo const* const	iface,
 
   switch (action) {
     case acREMOVE_ID	:
-      if (relay_opt!=0) DHCP_zeroOption(relay_opt);
+      if (relay_opt!=0) len = removeAgentOption(relay_opt, end_opt, len);
       break;
     case acADD_ID	:
       if (relay_opt==0) len = addAgentOption(iface, end_opt, len);
@@ -497,7 +525,7 @@ sendServerBcast(/*@in@*/struct ServerInfo const	* const		server,
     /*@modifies internalState@*/
 {
   struct DHCPllPacket		frame;
-  struct InterfaceInfo const	*iface = server->info.iface;
+  struct InterfaceInfo const	*iface = server->iface;
 
   memset(&frame, 0, sizeof frame);
   memset(frame.eth.ether_dhost, 255, sizeof frame.eth.ether_dhost);
@@ -524,10 +552,10 @@ sendServerUnicast(/*@in@*/struct ServerInfo const * const	server,
   memset(&sock, 0, sizeof sock);
   sock.sin_family = AF_INET;
 
-  sock.sin_addr = server->info.ip;
+  sock.sin_addr = server->info.unicast.ip;
   sock.sin_port = htons(DHCP_PORT_SERVER);
 	      
-  Wsendto(fds.sender_fd, buffer, size, 0,
+  Wsendto(server->info.unicast.fd, buffer, size, 0,
 	  reinterpret_cast(struct sockaddr *)(&sock),
 	  sizeof sock);
 }
@@ -631,14 +659,15 @@ execRelay()
     /*@globals fds, servers, internalState@*/
     /*@modifies internalState@*/
 {
-  size_t const			max_mtu   = determineMaxMTU();
-  size_t const			len_total = max_mtu + IFNAMSIZ + 4;
-  char				*buffer   = static_cast(char *)(Ealloca(len_total));
     /*@-sizeoftype@*/
   size_t const			szDHCPHDR = sizeof(struct DHCPHeader);
     /*@=sizeoftype@*/
+  size_t const			max_mtu   = determineMaxMTU();
+  size_t const			len_total = max_mtu + IFNAMSIZ + 4;
+  char				*buffer   = static_cast(char *)(alloca(len_total));
+
+  FatalErrnoError(buffer==0, 1, "alloca()");
 
-  assert(buffer!=0);
   assert(fds.dta!=0 || fds.len==0);
 
   while (true) {
@@ -667,7 +696,7 @@ execRelay()
       size = WrecvfromFlagsInet4(fdinfo->fd, buffer, len_total, &flags,
 				 &addr, &pkinfo);
 
-#ifdef ENABLE_LOGGING      
+#ifdef WITH_LOGGING
       logDHCPPackage(buffer, size, &pkinfo, &addr);
 #endif
       
